/*
 *    Digital Invisible Ink Toolkit
 *    Copyright (C) 2005  K. Hempstalk	
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package invisibleinktoolkit.stego;

import java.io.File;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import java.io.IOException;


/**
 * A cover image is an image that can hide a message.
 * <P>
 * A cover image can be any image that is on disk.  In this case
 * images that are not in a 24 bit colour format will be refused.
 * All the algorithms implemented in this toolkit rely solely on
 * 24 bit formats.
 * 
 * @author Kathryn Hempstalk.
 */
public class CoverImage{
	
	//CONSTRUCTORS
	
	/**
	 * Creates a new cover image, from a given path to the image.
	 *
	 * @param path The path to the image on disk.
	 * @throws IOException If there is a problem during reading
	 * of the file.
	 * @throws IllegalArgumentException If the file is not a 
	 * recognisable type.
	 * @throws NullPointerException If the path is null.
	 */
	public CoverImage(String path) throws IOException, 
	IllegalArgumentException,
	NullPointerException{
		
		//read in the image - appropriate reader will be 
		//automatically chosen from the registered list.
		mCover = ImageIO.read(new File(path));
		
		if(mCover == null){
			throw new IllegalArgumentException
			("File type is not a recognisable type.");
		}
		
		//check that it is a type we can deal with
		if(this.getLayerCount() <= 1 
				|| mCover.getType() == BufferedImage.TYPE_USHORT_555_RGB
				|| mCover.getType() == BufferedImage.TYPE_USHORT_565_RGB){
			throw new IllegalArgumentException
			("Picture colour depth is not deep enough!");
		}
		
	}
	
	
	//FUNCTIONS
	
	/**
	 * Gets the number of layers the image has.
	 * <P>
	 * The number of layers is determined by the colour depth of
	 * the image.  If a colour has a 24 bit colour depth, then the
	 * number of layers is 3, one for each of red, green and blue. In
	 * contrast, if the colour depth is 8, 1 layer is returned.
	 * <P>
	 * To summarise, 24 bits = 3 layers, 16 bit = 3 layers, 8 bit = 1.
	 * Images which do not have a deep enough set of colours return 0.
	 *
	 * @return The number of "layers" an image has.
	 */
	public int getLayerCount(){
		int type = mCover.getType();
		
		if (type == BufferedImage.TYPE_BYTE_BINARY)
			//1, 2 and 4 bit images
			return 0;
		else if (type == BufferedImage.TYPE_BYTE_INDEXED 
				|| type == BufferedImage.TYPE_BYTE_GRAY
				|| type == BufferedImage.TYPE_USHORT_GRAY)
			//8 bit images
			return 1;
		else
			//all other image types
			return 3;
	}
	
	
	/**
	 * Gets the image that is currently residing inside
	 * this cover.
	 *
	 * @return The image inside this cover.
	 */
	public BufferedImage getImage(){
		return mCover;
	}
	
	
	/**
	 * Sets the pixel bit at the given location
	 * to the new value.
	 *
	 * @param xpos The x position of the pixel.
	 * @param ypos The y position of the pixel.
	 * @param layer The layer of the bit.
	 * @param bitpos The position of the bit (0-7 or 0-4)
	 * @param newbit The new bit for the pixel.
	 * @throws IllegalArgumentException If the bit position is not
	 * in the right range, or the layer is incorrect.
	 */
	public void setPixelBit(int xpos, int ypos, int layer, int bitpos,
			boolean newbit) 
	throws IllegalArgumentException{
		
		//check the bit range
		if((mCover.getType() == BufferedImage.TYPE_USHORT_555_RGB
				|| mCover.getType() == BufferedImage.TYPE_USHORT_565_RGB )
				&& bitpos > 4)
			throw new IllegalArgumentException
			("Bit position in incorrect position for image (0-4)!");
		else if (bitpos > 7 || bitpos < 0)
			throw new IllegalArgumentException
			("Bit position in incorrect position for image (0-7)!");
		
		//check layer
		if(layer > this.getLayerCount() || layer < 0 )
			throw new IllegalArgumentException
			("Layer is incorrect for image type!");
		
		
		//get the pixel we want to work on
		int pixel = mCover.getRGB(xpos, ypos);
		
		int newcolour = 0, newpixel;
		
		//hide the bit
		if(newbit){
			newcolour = 1;
			newcolour = newcolour << (bitpos + (layer * 8));
			newpixel = pixel | newcolour;
		}else{
			newcolour = 0xfffffffe;
			for(int i = 0; i < (bitpos + (layer * 8)); i++){
				newcolour = (newcolour << 1) | 0x1;
			}
			newpixel = pixel & newcolour;		 
		}
		
		//now set the pixel.
		mCover.setRGB(xpos, ypos, newpixel);
		
	}
	
	
	//VARIABLES
	
	/**
	 * An image representation of the cover.
	 */
	private BufferedImage mCover;
	
	
}
//end of class.
